@@ -8,8 +8,8 @@
 #include "memory-util.h"
 #include "parse-util.h"
 //#include "process-util.h"
-#include "procfs-util.h"
-#include "string-util.h"
+//#include "procfs-util.h"
+// #include "string-util.h"
 
 /// Additional includes needed by elogind
 
@@ -20,7 +20,7 @@ uint64_t physical_memory(void) {
         long sc;
         int r;
 
-        /* We return this as uint64_t in case we are running as 32bit process on a 64bit kernel with huge amounts of
+        /* We return this as uint64_t in case we are running as 32-bit process on a 64-bit kernel with huge amounts of
          * memory.
          *
          * In order to support containers nicely that have a configured memory limit we'll take the minimum of the
@@ -129,7 +129,7 @@ uint64_t system_tasks_max(void) {
          *
          *    This limits the numeric range PIDs can take, and thus indirectly also limits the number of
          *    concurrent threads. It's primarily a compatibility concept: some crappy old code used a signed
-         *    16bit type for PIDs, hence the kernel provides a way to ensure the PIDs never go beyond
+         *    16-bit type for PIDs, hence the kernel provides a way to ensure the PIDs never go beyond
          *    INT16_MAX by default.
          *
          *    Also note the weird definition: PIDs assigned will be kept below this value, which means
