@@ -35,10 +35,8 @@ int get_locales(char ***l);
 bool locale_is_valid(const char *name);
 int locale_is_installed(const char *name);
 
-#define _(String) gettext(String)
+#define _(String) dgettext(GETTEXT_PACKAGE, String)
 #define N_(String) String
-#if 0 /// UNNEEDED by elogind
-#endif // 0
 
 bool is_locale_utf8(void);
 
