@@ -22,6 +22,7 @@
 
 #include "macro.h"
 #include "missing_keyctl.h"
+#include "missing_sched.h"
 #include "missing_stat.h"
 #include "missing_syscall_def.h"
 
@@ -669,6 +670,22 @@ static inline ssize_t missing_getdents64(int fd, void *buffer, size_t length) {
 
 /* ======================================================================= */
 
+#if !HAVE_SCHED_SETATTR
+
+static inline ssize_t missing_sched_setattr(pid_t pid, struct sched_attr *attr, unsigned int flags) {
+#  if defined __NR_sched_setattr
+        return syscall(__NR_sched_setattr, pid, attr, flags);
+#  else
+        errno = ENOSYS;
+        return -1;
+#  endif
+}
+
+#  define sched_setattr missing_sched_setattr
+#endif
+
+/* ======================================================================= */
+
 /* glibc does not provide clone() on ia64, only clone2(). Not only that, but it also doesn't provide a
  * prototype, only the symbol in the shared library (it provides a prototype for clone(), but not the
  * symbol in the shared library). */
