@@ -11,9 +11,10 @@
 //#include "errno-util.h"
 #include "fd-util.h"
 #include "macro.h"
-//#include "missing_syscall.h"
-//#include "sparse-endian.h"
-//#include "stat-util.h"
+#include "missing_syscall.h"
+#include "parse-util.h"
+#include "sparse-endian.h"
+#include "stat-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
 //#include "time-util.h"
@@ -26,7 +27,7 @@ int getxattr_at_malloc(
                 int flags,
                 char **ret) {
 
-        _cleanup_close_ int opened_fd = -1;
+        _cleanup_close_ int opened_fd = -EBADF;
         unsigned n_attempts = 7;
         bool by_procfs = false;
         size_t l = 100;
@@ -118,6 +119,20 @@ int getxattr_at_malloc(
         }
 }
 
+int getxattr_at_bool(int fd, const char *path, const char *name, int flags) {
+        _cleanup_free_ char *v = NULL;
+        int r;
+
+        r = getxattr_at_malloc(fd, path, name, flags, &v);
+        if (r < 0)
+                return r;
+
+        if (memchr(v, 0, r)) /* Refuse embedded NUL byte */
+                return -EINVAL;
+
+        return parse_boolean(v);
+}
+
 #if 0 /// UNNEEDED by elogind
 static int parse_crtime(le64_t le, usec_t *usec) {
         uint64_t u;
@@ -153,9 +168,9 @@ int fd_getcrtime_at(
         /* So here's the deal: the creation/birth time (crtime/btime) of a file is a relatively newly supported concept
          * on Linux (or more strictly speaking: a concept that only recently got supported in the API, it was
          * implemented on various file systems on the lower level since a while, but never was accessible). However, we
-         * needed a concept like that for vaccuuming algorithms and such, hence we emulated it via a user xattr for a
+         * needed a concept like that for vacuuming algorithms and such, hence we emulated it via a user xattr for a
          * long time. Starting with Linux 4.11 there's statx() which exposes the timestamp to userspace for the first
-         * time, where it is available. Thius function will read it, but it tries to keep some compatibility with older
+         * time, where it is available. This function will read it, but it tries to keep some compatibility with older
          * systems: we try to read both the crtime/btime and the xattr, and then use whatever is older. After all the
          * concept is useful for determining how "old" a file really is, and hence using the older of the two makes
          * most sense. */
@@ -213,7 +228,7 @@ int listxattr_at_malloc(
                 int flags,
                 char **ret) {
 
-        _cleanup_close_ int opened_fd = -1;
+        _cleanup_close_ int opened_fd = -EBADF;
         bool by_procfs = false;
         unsigned n_attempts = 7;
         size_t l = 100;
@@ -295,3 +310,72 @@ int listxattr_at_malloc(
         }
 }
 #endif // 0
+
+int xsetxattr(int fd,
+              const char *path,
+              const char *name,
+              const char *value,
+              size_t size,
+              int flags) {
+
+        _cleanup_close_ int opened_fd = -EBADF;
+        bool by_procfs = false;
+        int r;
+
+        assert(fd >= 0 || fd == AT_FDCWD);
+        assert(name);
+        assert(value);
+        assert((flags & ~(AT_SYMLINK_FOLLOW|AT_EMPTY_PATH)) == 0);
+
+        /* So, this is a single function that does what setxattr()/lsetxattr()/fsetxattr() do, but in one go,
+         * and with additional bells and whistles. Specifically:
+         *
+         * 1. This works on O_PATH fds (which fsetxattr() does not)
+         * 2. Provides full openat()-style semantics, i.e. by-fd, by-path and combination thereof
+         * 3. As extension to openat()-style semantics implies AT_EMPTY_PATH if path is NULL.
+         */
+
+        if (!path) /* If path is NULL, imply AT_EMPTY_PATH. – But if it's "", don't — for safety reasons. */
+                flags |= AT_EMPTY_PATH;
+
+        if (size == SIZE_MAX)
+                size = strlen(value);
+
+        if (isempty(path)) {
+                if (!FLAGS_SET(flags, AT_EMPTY_PATH))
+                        return -EINVAL;
+
+                if (fd == AT_FDCWD) /* Both unspecified? Then operate on current working directory */
+                        path = ".";
+                else {
+                        r = fd_is_opath(fd);
+                        if (r < 0)
+                                return r;
+
+                        by_procfs = r;
+                        path = NULL;
+                }
+
+        } else if (fd != AT_FDCWD) {
+
+                /* If both have been specified, then we go via O_PATH */
+                opened_fd = openat(fd, path, O_PATH|O_CLOEXEC|(FLAGS_SET(flags, AT_SYMLINK_FOLLOW) ? 0 : O_NOFOLLOW));
+                if (opened_fd < 0)
+                        return -errno;
+
+                fd = opened_fd;
+                path = NULL;
+                by_procfs = true; /* fsetxattr() is not going to work, go via /proc/ link right-away */
+        }
+
+        if (path)
+                r = FLAGS_SET(flags, AT_SYMLINK_FOLLOW) ? setxattr(path, name, value, size, 0)
+                                                        : lsetxattr(path, name, value, size, 0);
+        else
+                r = by_procfs ? setxattr(FORMAT_PROC_FD_PATH(fd), name, value, size, 0)
+                              : fsetxattr(fd, name, value, size, 0);
+        if (r < 0)
+                return -errno;
+
+        return 0;
+}
