@@ -7,6 +7,7 @@
 #include "fd-util.h"
 //#include "fileio.h"
 #include "hexdecoct.h"
+// #include "memstream-util.h"
 #include "sort-util.h"
 #include "string-util.h"
 #include "strv.h"
@@ -692,8 +693,8 @@ static int match_component_compare(const struct bus_match_component *a, const st
         return CMP(a->type, b->type);
 }
 
-void bus_match_parse_free(struct bus_match_component *components, unsigned n_components) {
-        for (unsigned i = 0; i < n_components; i++)
+void bus_match_parse_free(struct bus_match_component *components, size_t n_components) {
+        for (size_t i = 0; i < n_components; i++)
                 free(components[i].value_str);
 
         free(components);
@@ -702,20 +703,22 @@ void bus_match_parse_free(struct bus_match_component *components, unsigned n_com
 int bus_match_parse(
                 const char *match,
                 struct bus_match_component **ret_components,
-                unsigned *ret_n_components) {
+                size_t *ret_n_components) {
 
         struct bus_match_component *components = NULL;
-        unsigned n_components = 0;
+        size_t n_components = 0;
         int r;
 
         assert(match);
         assert(ret_components);
         assert(ret_n_components);
 
+        CLEANUP_ARRAY(components, n_components, bus_match_parse_free);
+
         while (*match != '\0') {
                 const char *eq, *q;
                 enum bus_match_node_type t;
-                unsigned j = 0;
+                size_t j = 0;
                 _cleanup_free_ char *value = NULL;
                 bool escaped = false, quoted;
                 uint8_t u;
@@ -724,16 +727,12 @@ int bus_match_parse(
                 match += strspn(match, " ");
 
                 eq = strchr(match, '=');
-                if (!eq) {
-                        r = -EINVAL;
-                        goto fail;
-                }
+                if (!eq)
+                        return -EINVAL;
 
                 t = bus_match_node_type_from_string(match, eq - match);
-                if (t < 0) {
-                        r = -EINVAL;
-                        goto fail;
-                }
+                if (t < 0)
+                        return -EINVAL;
 
                 quoted = eq[1] == '\'';
 
@@ -741,14 +740,12 @@ int bus_match_parse(
 
                         if (*q == '\0') {
 
-                                if (quoted) {
-                                        r = -EINVAL;
-                                        goto fail;
-                                } else {
-                                        if (value)
-                                                value[j] = '\0';
-                                        break;
-                                }
+                                if (quoted)
+                                        return -EINVAL;
+
+                                if (value)
+                                        value[j] = '\0';
+                                break;
                         }
 
                         if (!escaped) {
@@ -772,10 +769,8 @@ int bus_match_parse(
                                 }
                         }
 
-                        if (!GREEDY_REALLOC(value, j + 2)) {
-                                r = -ENOMEM;
-                                goto fail;
-                        }
+                        if (!GREEDY_REALLOC(value, j + 2))
+                                return -ENOMEM;
 
                         value[j++] = *q;
                         escaped = false;
@@ -783,25 +778,21 @@ int bus_match_parse(
 
                 if (!value) {
                         value = strdup("");
-                        if (!value) {
-                                r = -ENOMEM;
-                                goto fail;
-                        }
+                        if (!value)
+                                return -ENOMEM;
                 }
 
                 if (t == BUS_MATCH_MESSAGE_TYPE) {
                         r = bus_message_type_from_string(value, &u);
                         if (r < 0)
-                                goto fail;
+                                return r;
 
                         value = mfree(value);
                 } else
                         u = 0;
 
-                if (!GREEDY_REALLOC(components, n_components + 1)) {
-                        r = -ENOMEM;
-                        goto fail;
-                }
+                if (!GREEDY_REALLOC(components, n_components + 1))
+                        return -ENOMEM;
 
                 components[n_components++] = (struct bus_match_component) {
                         .type = t,
@@ -812,10 +803,8 @@ int bus_match_parse(
                 if (q[quoted] == 0)
                         break;
 
-                if (q[quoted] != ',') {
-                        r = -EINVAL;
-                        goto fail;
-                }
+                if (q[quoted] != ',')
+                        return -EINVAL;
 
                 match = q + 1 + quoted;
         }
@@ -824,26 +813,20 @@ int bus_match_parse(
         typesafe_qsort(components, n_components, match_component_compare);
 
         /* Check for duplicates */
-        for (unsigned i = 0; i+1 < n_components; i++)
-                if (components[i].type == components[i+1].type) {
-                        r = -EINVAL;
-                        goto fail;
-                }
+        for (size_t i = 0; i+1 < n_components; i++)
+                if (components[i].type == components[i+1].type)
+                        return -EINVAL;
 
-        *ret_components = components;
+        *ret_components = TAKE_PTR(components);
         *ret_n_components = n_components;
 
         return 0;
-
-fail:
-        bus_match_parse_free(components, n_components);
-        return r;
 }
 
 #if 0 /// UNNEEDED by elogind
-char *bus_match_to_string(struct bus_match_component *components, unsigned n_components) {
-        _cleanup_free_ char *buffer = NULL;
-        size_t size = 0;
+char *bus_match_to_string(struct bus_match_component *components, size_t n_components) {
+        _cleanup_(memstream_done) MemStream m = {};
+        FILE *f;
         int r;
 
         if (n_components <= 0)
@@ -851,11 +834,11 @@ char *bus_match_to_string(struct bus_match_component *components, unsigned n_com
 
         assert(components);
 
-        FILE *f = open_memstream_unlocked(&buffer, &size);
+        f = memstream_init(&m);
         if (!f)
                 return NULL;
 
-        for (unsigned i = 0; i < n_components; i++) {
+        for (size_t i = 0; i < n_components; i++) {
                 char buf[32];
 
                 if (i != 0)
@@ -873,18 +856,19 @@ char *bus_match_to_string(struct bus_match_component *components, unsigned n_com
                 fputc('\'', f);
         }
 
-        r = fflush_and_check(f);
-        safe_fclose(f);
+        char *buffer;
+        r = memstream_finalize(&m, &buffer, NULL);
         if (r < 0)
                 return NULL;
-        return TAKE_PTR(buffer);
+
+        return buffer;
 }
 #endif // 0
 
 int bus_match_add(
                 struct bus_match_node *root,
                 struct bus_match_component *components,
-                unsigned n_components,
+                size_t n_components,
                 struct match_callback *callback) {
 
         int r;
@@ -892,7 +876,7 @@ int bus_match_add(
         assert(root);
         assert(callback);
 
-        for (unsigned i = 0; i < n_components; i++) {
+        for (size_t i = 0; i < n_components; i++) {
                 r = bus_match_add_compare_value(root,
                                                 components[i].type,
                                                 components[i].value_u8,
@@ -1040,7 +1024,7 @@ void bus_match_dump(FILE *out, struct bus_match_node *node, unsigned level) {
                 bus_match_dump(out, c, level + 1);
 }
 
-enum bus_match_scope bus_match_get_scope(const struct bus_match_component *components, unsigned n_components) {
+enum bus_match_scope bus_match_get_scope(const struct bus_match_component *components, size_t n_components) {
         bool found_driver = false;
 
         if (n_components <= 0)
@@ -1054,7 +1038,7 @@ enum bus_match_scope bus_match_get_scope(const struct bus_match_component *compo
          * local messages, then we check if it only matches on the
          * driver. */
 
-        for (unsigned i = 0; i < n_components; i++) {
+        for (size_t i = 0; i < n_components; i++) {
                 const struct bus_match_component *c = components + i;
 
                 if (c->type == BUS_MATCH_SENDER) {
