@@ -20,8 +20,14 @@ int device_new_from_strv(sd_device **ret, char **strv);
 int device_opendir(sd_device *device, const char *subdir, DIR **ret);
 
 int device_get_property_bool(sd_device *device, const char *key);
+#if 0 /// UNNEEDED by elogind
+int device_get_property_int(sd_device *device, const char *key, int *ret);
+#endif // 0
 int device_get_sysattr_int(sd_device *device, const char *sysattr, int *ret_value);
-int device_get_sysattr_unsigned(sd_device *device, const char *sysattr, unsigned *ret_value);
+int device_get_sysattr_unsigned_full(sd_device *device, const char *sysattr, unsigned base, unsigned *ret_value);
+static inline int device_get_sysattr_unsigned(sd_device *device, const char *sysattr, unsigned *ret_value) {
+        return device_get_sysattr_unsigned_full(device, sysattr, 0, ret_value);
+}
 int device_get_sysattr_bool(sd_device *device, const char *sysattr);
 int device_get_device_id(sd_device *device, const char **ret);
 
@@ -47,6 +53,7 @@ int device_ensure_usec_initialized(sd_device *device, sd_device *device_old);
 #endif // 0
 int device_add_devlink(sd_device *device, const char *devlink);
 #if 0 /// UNNEEDED by elogind
+int device_remove_devlink(sd_device *device, const char *devlink);
 bool device_has_devlink(sd_device *device, const char *devlink);
 #endif // 0
 int device_add_property(sd_device *device, const char *property, const char *value);
@@ -69,7 +76,6 @@ int device_get_properties_nulstr(sd_device *device, const char **ret_nulstr, siz
 #if 0 /// UNNEEDED by elogind
 int device_get_properties_strv(sd_device *device, char ***ret);
 
-int device_rename(sd_device *device, const char *name);
 int device_clone_with_db(sd_device *device, sd_device **ret);
 
 int device_tag_index(sd_device *dev, sd_device *dev_old, bool add);
