@@ -25,6 +25,7 @@
 #include "fd-util.h"
 #include "fileio.h"
 #include "fs-util.h"
+#include "logind-dbus.h"
 #include "mount-setup.h"
 #include "musl_missing.h"
 #include "parse-util.h"
@@ -80,6 +81,7 @@ static int elogind_sigchld_handler(
                 sd_event_source* s,
                 const struct signalfd_siginfo* si,
                 void* userdata ) {
+        const HandleActionData* a;
         Manager* m = userdata;
         int r, status;
 
@@ -96,10 +98,16 @@ static int elogind_sigchld_handler(
 
                 /* The sleep forker PID is always "the outer one", so wait for it second. */
                 if ( m->sleep_fork_pid > 0 ) {
+                        a = m->sleep_fork_action;
                         waitpid(m->sleep_fork_pid, &status, WNOHANG | WUNTRACED);
                         log_debug_elogind( "sleep_fork PID %d waitpid() set status %d", m->sleep_fork_pid, status );
                         if ( WIFEXITED(status) || WIFSIGNALED(status) )
                                 m->sleep_fork_pid = 0;
+                        /* Tell people that they now may take a lock again */
+                        if ( a && a->sleep_operation != _SLEEP_OPERATION_INVALID ) {
+                                (void) send_prepare_for( m, a, false );
+                                m->sleep_fork_action = NULL; /* All done */
+                        }
                 }
         }
 
@@ -416,12 +424,13 @@ void elogind_manager_free( Manager* m ) {
 int elogind_manager_new( Manager* m ) {
         int r = 0;
 
-        m->cgroups_agent_fd = -1;
-        m->pin_cgroupfs_fd  = -1;
-        m->test_run_flags   = 0;
-        m->do_interrupt     = false;
-        m->sleep_fork_pid   = 0;
-        m->tool_fork_pid    = 0;
+        m->cgroups_agent_fd  = -1;
+        m->pin_cgroupfs_fd   = -1;
+        m->test_run_flags    = 0;
+        m->do_interrupt      = false;
+        m->sleep_fork_pid    = 0;
+        m->tool_fork_pid     = 0;
+        m->sleep_fork_action = NULL;
 
         /* Init poweroff/suspend interruption */
         m->allow_poweroff_interrupts     = false;
