@@ -5,7 +5,7 @@
 /***
   This file is part of elogind.
 
-  Copyright 2017-2018 Sven Eden
+  Copyright 2017 Sven Eden
 
   elogind is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,7 @@
 ***/
 
 
-#include "elogind-dbus.h"
+#include "logind-dbus.h"
 #include "logind.h"
 #include "sd-bus.h"
 
@@ -39,9 +39,6 @@ void elogind_manager_free(Manager* m);
 /// Add-On for manager_new()
 int elogind_manager_new(Manager* m);
 
-/** Add-On for manager_reset_config() */
-void elogind_manager_reset_config(Manager* m);
-
 /// Add-On for manager_startup()
 int elogind_manager_startup(Manager* m);
 
