@@ -11,9 +11,9 @@
 #include "async.h"
 #include "fd-util.h"
 #include "logind-button.h"
+#include "logind-dbus.h"
 #include "missing_input.h"
 #include "string-util.h"
-#include "util.h"
 
 #define CONST_MAX5(a, b, c, d, e) CONST_MAX(CONST_MAX(a, b), CONST_MAX(CONST_MAX(c, d), e))
 
@@ -49,13 +49,14 @@ Button* button_new(Manager *m, const char *name) {
         }
 
         b->manager = m;
-        b->fd = -1;
+        b->fd = -EBADF;
 
         return b;
 }
 
-void button_free(Button *b) {
-        assert(b);
+Button *button_free(Button *b) {
+        if (!b)
+                return NULL;
 
         hashmap_remove(b->manager->buttons, b->name);
 
@@ -66,7 +67,8 @@ void button_free(Button *b) {
 
         free(b->name);
         free(b->seat);
-        free(b);
+
+        return mfree(b);
 }
 
 int button_set_seat(Button *b, const char *sn) {
@@ -342,6 +344,7 @@ static int button_dispatch(sd_event_source *s, int fd, uint32_t revents, void *u
                         b->lid_closed = true;
                         button_lid_switch_handle_action(b->manager, true);
                         button_install_check_event_source(b);
+                        manager_send_changed(b->manager, "LidClosed", NULL);
 
                 } else if (ev.code == SW_DOCK) {
                         log_struct(LOG_INFO,
@@ -360,6 +363,7 @@ static int button_dispatch(sd_event_source *s, int fd, uint32_t revents, void *u
 
                         b->lid_closed = false;
                         b->check_event_source = sd_event_source_unref(b->check_event_source);
+                        manager_send_changed(b->manager, "LidClosed", NULL);
 
                 } else if (ev.code == SW_DOCK) {
                         log_struct(LOG_INFO,
@@ -464,7 +468,7 @@ static int button_set_mask(const char *name, int fd) {
 }
 
 int button_open(Button *b) {
-        _cleanup_(asynchronous_closep) int fd = -1;
+        _cleanup_(asynchronous_closep) int fd = -EBADF;
         const char *p;
         char name[256];
         int r;
@@ -513,6 +517,7 @@ int button_check_switches(Button *b) {
 
         b->lid_closed = bitset_get(switches, SW_LID);
         b->docked = bitset_get(switches, SW_DOCK);
+        manager_send_changed(b->manager, "LidClosed", NULL);
 
         if (b->lid_closed)
                 button_install_check_event_source(b);
