@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
 typedef enum DeviceType DeviceType;
@@ -28,7 +28,9 @@ struct SessionDevice {
 };
 
 int session_device_new(Session *s, dev_t dev, bool open_device, SessionDevice **out);
-void session_device_free(SessionDevice *sd);
+SessionDevice *session_device_free(SessionDevice *sd);
+DEFINE_TRIVIAL_CLEANUP_FUNC(SessionDevice*, session_device_free);
+
 void session_device_complete_pause(SessionDevice *sd);
 
 void session_device_resume_all(Session *s);
