@@ -42,6 +42,7 @@
 /// Additional includes needed by elogind
 #include "cgroup-setup.h"
 #include "extract-word.h"
+#include "musl_missing.h"
 
 #define RELEASE_USEC (20*USEC_PER_SEC)
 
@@ -720,6 +721,7 @@ static int session_start_scope(Session *s, sd_bus_message *properties, sd_bus_er
                                 s->manager,
                                 scope,
                                 &s->leader,
+                                /* allow_pidfd = */ true,
                                 s->user->slice,
                                 description,
                                 /* These two have StopWhenUnneeded= set, hence add a dep towards them */
@@ -1463,28 +1465,25 @@ SessionState session_get_state(Session *s) {
 int session_kill(Session *s, KillWho who, int signo) {
         assert(s);
 
-#if 0 /// Without direct cgroup support, elogind can not kill sessions
-        if (!s->scope)
-                return -ESRCH;
+        switch (who) {
 
-        return manager_kill_unit(s->manager, s->scope, who, signo, NULL);
-#else // 0
-        if (who == KILL_LEADER) {
-                if (s->leader.pid <= 0)
+        case KILL_ALL:
+#if 0 /// elogind does not use units
+                if (!s->scope)
                         return -ESRCH;
 
-                /* FIXME: verify that leader is in cgroup?  */
-
-                if (kill(s->leader.pid, signo) < 0) {
-                        return log_error_errno(errno, "Failed to kill process leader %d for session %s: %m", s->leader.pid, s->id);
-                }
-                return 0;
-        } else
-                // const char *path, int sig, CGroupFlags flags, Set *s, cg_kill_log_func_t kill_log, void *userdata
+                return manager_kill_unit(s->manager, s->scope, KILL_ALL, signo, NULL);
+#else // 0
                 return cg_kill_recursive (SYSTEMD_CGROUP_CONTROLLER, signo,
                                           CGROUP_IGNORE_SELF | CGROUP_REMOVE,
                                           NULL, NULL, NULL);
 #endif // 0
+        case KILL_LEADER:
+                return pidref_kill(&s->leader, signo);
+
+        default:
+                assert_not_reached();
+        }
 }
 
 static int session_open_vt(Session *s, bool reopen) {
