@@ -1,10 +1,13 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "logind-action.h"
 #include "logind-session.h"
 #include "test-tables.h"
+#include "tests.h"
 
 int main(int argc, char **argv) {
+        test_setup_logging(LOG_DEBUG);
+
         test_table(handle_action, HANDLE_ACTION);
         test_table(inhibit_mode, INHIBIT_MODE);
         test_table(kill_who, KILL_WHO);
