@@ -6,11 +6,12 @@
 #include "sd-bus.h"
 
 #include "bus-error.h"
-#include "dev-setup.h"
+// #include "bus-locator.h"
+//#include "dev-setup.h"
 #include "format-util.h"
 #include "fs-util.h"
-#include "label.h"
-#include "limits-util.h"
+#include "label-util.h"
+// #include "limits-util.h"
 //#include "main-func.h"
 #include "mkdir-label.h"
 #include "mount-util.h"
@@ -20,7 +21,7 @@
 //#include "selinux-util.h"
 #include "smack-util.h"
 #include "stdio-util.h"
-#include "string-util.h"
+// #include "string-util.h"
 //#include "strv.h"
 #include "user-util.h"
 /// Additional includes needed by elogind
@@ -37,13 +38,13 @@ static int acquire_runtime_dir_properties(uint64_t *size, uint64_t *inodes) {
         if (r < 0)
                 return log_error_errno(r, "Failed to connect to system bus: %m");
 
-        r = sd_bus_get_property_trivial(bus, "org.freedesktop.login1", "/org/freedesktop/login1", "org.freedesktop.login1.Manager", "RuntimeDirectorySize", &error, 't', size);
+        r = bus_get_property_trivial(bus, bus_login_mgr, "RuntimeDirectorySize", &error, 't', size);
         if (r < 0) {
                 log_warning_errno(r, "Failed to acquire runtime directory size, ignoring: %s", bus_error_message(&error, r));
                 *size = physical_memory_scale(10U, 100U); /* 10% */
         }
 
-        r = sd_bus_get_property_trivial(bus, "org.freedesktop.login1", "/org/freedesktop/login1", "org.freedesktop.login1.Manager", "RuntimeDirectoryInodesMax", &error, 't', inodes);
+        r = bus_get_property_trivial(bus, bus_login_mgr, "RuntimeDirectoryInodesMax", &error, 't', inodes);
         if (r < 0) {
                 log_warning_errno(r, "Failed to acquire number of inodes for runtime directory, ignoring: %s", bus_error_message(&error, r));
                 *inodes = DIV_ROUND_UP(*size, 4096);
@@ -71,7 +72,7 @@ static int user_mkdir_runtime_path(
         if (r < 0)
                 return log_error_errno(r, "Failed to create /run/user: %m");
 
-        if (path_is_mount_point(runtime_path, NULL, 0) >= 0)
+        if (path_is_mount_point(runtime_path, NULL, 0) > 0)
                 log_debug("%s is already a mount point", runtime_path);
         else {
                 char options[sizeof("mode=0700,uid=,gid=,size=,nr_inodes=,smackfsroot=*")
@@ -215,7 +216,7 @@ static int run(int argc, char *argv[]) {
 
         umask(0022);
 
-        r = mac_selinux_init();
+        r = mac_init();
         if (r < 0)
                 return r;
 
