@@ -42,6 +42,12 @@ const BusLocator* const bus_network_mgr = &(BusLocator){
         .interface = "org.freedesktop.network1.Manager"
 };
 
+const BusLocator* const bus_oom_mgr = &(BusLocator){
+        .destination = "org.freedesktop.oom1",
+        .path = "/org/freedesktop/oom1",
+        .interface = "org.freedesktop.oom1.Manager"
+};
+
 const BusLocator* const bus_portable_mgr = &(BusLocator){
         .destination = "org.freedesktop.portable1",
         .path = "/org/freedesktop/portable1",
@@ -67,6 +73,12 @@ const BusLocator* const bus_timedate = &(BusLocator){
 };
 #endif // 0
 
+const BusLocator* const bus_timesync_mgr = &(BusLocator){
+        .destination = "org.freedesktop.timesync1",
+        .path = "/org/freedesktop/timesync1",
+        .interface = "org.freedesktop.timesync1.Manager"
+};
+
 const BusLocator* const bus_hostname = &(BusLocator){
         .destination = "org.freedesktop.hostname1",
         .path = "/org/freedesktop/hostname1",
@@ -129,6 +141,7 @@ int bus_get_property(
 
         return sd_bus_get_property(bus, locator->destination, locator->path, locator->interface, member, error, reply, type);
 }
+#endif // 0
 
 int bus_get_property_trivial(
                 sd_bus *bus,
@@ -141,7 +154,6 @@ int bus_get_property_trivial(
 
         return sd_bus_get_property_trivial(bus, locator->destination, locator->path, locator->interface, member, error, type, ptr);
 }
-#endif // 0
 
 int bus_get_property_string(
                 sd_bus *bus,
