@@ -11,6 +11,7 @@ typedef struct BusLocator {
 
 #if 0 /// UNNEEDED by elogind
 extern const BusLocator* const bus_home_mgr;
+extern const BusLocator* const bus_hostname;
 extern const BusLocator* const bus_import_mgr;
 extern const BusLocator* const bus_locale;
 #endif // 0
@@ -18,12 +19,13 @@ extern const BusLocator* const bus_login_mgr;
 #if 0 /// UNNEEDED by elogind
 extern const BusLocator* const bus_machine_mgr;
 extern const BusLocator* const bus_network_mgr;
+extern const BusLocator* const bus_oom_mgr;
 extern const BusLocator* const bus_portable_mgr;
 extern const BusLocator* const bus_resolve_mgr;
 extern const BusLocator* const bus_systemd_mgr;
 extern const BusLocator* const bus_timedate;
 #endif // 0
-extern const BusLocator* const bus_hostname;
+extern const BusLocator* const bus_timesync_mgr;
 
 /* Shorthand flavors of the sd-bus convenience helpers with destination,path,interface strings encapsulated
  * within a single struct. */
@@ -31,8 +33,8 @@ int bus_call_method_async(sd_bus *bus, sd_bus_slot **slot, const BusLocator *loc
 int bus_call_method(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, sd_bus_message **reply, const char *types, ...);
 #if 0 /// UNNEEDED by elogind
 int bus_get_property(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, sd_bus_message **reply, const char *type);
-int bus_get_property_trivial(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, char type, void *ptr);
 #endif // 0
+int bus_get_property_trivial(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, char type, void *ptr);
 int bus_get_property_string(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, char **ret);
 #if 0 /// UNNEEDED by elogind
 int bus_get_property_strv(sd_bus *bus, const BusLocator *locator, const char *member, sd_bus_error *error, char ***ret);
