@@ -11,6 +11,8 @@
 
 #include "errno-util.h"
 #include "macro.h"
+#include "runtime-scope.h"
+#include "set.h"
 #include "string-util.h"
 #include "time-util.h"
 
@@ -43,10 +45,8 @@ int bus_connect_system_systemd(sd_bus **ret_bus);
 int bus_connect_user_systemd(sd_bus **ret_bus);
 #endif // 0
 
-int bus_connect_transport(BusTransport transport, const char *host, bool user, sd_bus **bus);
-#if 0 /// UNNEEDED by elogind
-int bus_connect_transport_systemd(BusTransport transport, const char *host, bool user, sd_bus **bus);
-#endif // 0
+int bus_connect_transport(BusTransport transport, const char *host, RuntimeScope runtime_scope, sd_bus **bus);
+int bus_connect_transport_elogind(BusTransport transport, const char *host, RuntimeScope runtime_scope, sd_bus **bus);
 
 int bus_log_address_error(int r, BusTransport transport);
 int bus_log_connect_error(int r, BusTransport transport);
@@ -54,6 +54,9 @@ int bus_log_connect_error(int r, BusTransport transport);
 #define bus_log_parse_error(r)                                  \
         log_error_errno(r, "Failed to parse bus message: %m")
 
+#define bus_log_parse_error_debug(r)                            \
+        log_debug_errno(r, "Failed to parse bus message: %m")
+
 #define bus_log_create_error(r)                                 \
         log_error_errno(r, "Failed to create bus message: %m")
 
@@ -71,6 +74,15 @@ static inline int bus_open_system_watch_bind(sd_bus **ret) {
 }
 
 int bus_reply_pair_array(sd_bus_message *m, char **l);
+
+/* Listen to GetMallocInfo() calls to 'destination' and return malloc_info() via FD */
+int bus_register_malloc_status(sd_bus *bus, const char *destination);
 #endif // 0
 
 extern const struct hash_ops bus_message_hash_ops;
+
+int bus_message_append_string_set(sd_bus_message *m, Set *s);
+
+#if 0 /// UNNEEDED by elogind
+int bus_property_get_string_set(sd_bus *bus, const char *path, const char *interface, const char *property, sd_bus_message *reply, void *userdata, sd_bus_error *error);
+#endif // 0
