@@ -27,6 +27,11 @@ int cg_attach_fallback(const char *controller, const char *path, pid_t pid);
 #endif // 0
 int cg_create_and_attach(const char *controller, const char *path, pid_t pid);
 
+int cg_set_access(const char *controller, const char *path, uid_t uid, gid_t gid);
+#if 0 /// UNNEEDED by elogind
+int cg_set_access_recursive(const char *controller, const char *path, uid_t uid, gid_t gid);
+#endif // 0
+
 int cg_migrate(const char *cfrom, const char *pfrom, const char *cto, const char *pto, CGroupFlags flags);
 int cg_migrate_recursive(const char *cfrom, const char *pfrom, const char *cto, const char *pto, CGroupFlags flags);
 #if 0 /// UNNEEDED by elogind
