@@ -159,7 +159,11 @@ static int next_assignment(
         /* Warn about unknown non-extension fields. */
         if (!(flags & CONFIG_PARSE_RELAXED) && !startswith(lvalue, "X-"))
                 log_syntax(unit, LOG_WARNING, filename, line, 0,
-                           "Unknown key name '%s' in section '%s', ignoring.", lvalue, section);
+                           "Unknown key '%s'%s%s%s, ignoring.",
+                           lvalue,
+                           section ? " in section [" : "",
+                           strempty(section),
+                           section ? "]" : "");
 
         return 0;
 }
@@ -760,6 +764,10 @@ int config_get_stats_by_path(
                         return -errno;
                 }
 
+                /* Skipping an empty file. */
+                if (null_or_empty(&st))
+                        continue;
+
                 r = hashmap_put_stats_by_path(&stats_by_path, *f, &st);
                 if (r < 0)
                         return r;
