@@ -24,11 +24,14 @@ int terminal_urlify_man(const char *page, const char *section, char **ret);
 
 #if 0 /// UNNEEDED by elogind
 typedef enum CatFlags {
-        CAT_FLAGS_MAIN_FILE_OPTIONAL = 1 << 0,
+        CAT_CONFIG_OFF          = 0,
+        CAT_CONFIG_ON           = 1 << 0,
+        CAT_FORMAT_HAS_SECTIONS = 1 << 1,  /* Sections are meaningful for this file format */
+        CAT_TLDR                = 1 << 2,  /* Only print comments and relevant section headers */
 } CatFlags;
 
 int cat_files(const char *file, char **dropins, CatFlags flags);
-int conf_files_cat(const char *root, const char *name);
+int conf_files_cat(const char *root, const char *name, CatFlags flags);
 #endif // 0
 
 #define RED_CROSS_MARK_MAX (STRLEN(ANSI_HIGHLIGHT_RED) + STRLEN("✗") + STRLEN(ANSI_NORMAL) + 1)
