@@ -1,8 +1,8 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
-//#include <stdint.h>
-//#include <sys/ioctl.h>
+#include <stdint.h>
+#include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <unistd.h>
 
@@ -14,8 +14,8 @@
 #endif
 
 #include "alloc-util.h"
-//#include "errno-util.h"
-//#include "fd-util.h"
+#include "errno-util.h"
+#include "fd-util.h"
 #include "fileio.h"
 #include "log.h"
 //#include "proc-cmdline.h"
@@ -23,7 +23,14 @@
 #include "reboot-util.h"
 #include "string-util.h"
 #include "umask-util.h"
-//#include "virt.h"
+#include "utf8.h"
+#include "virt.h"
+
+bool reboot_parameter_is_valid(const char *parameter) {
+        assert(parameter);
+
+        return ascii_is_valid(parameter) && strlen(parameter) <= NAME_MAX;
+}
 
 int update_reboot_parameter_and_warn(const char *parameter, bool keep) {
         int r;
@@ -42,6 +49,9 @@ int update_reboot_parameter_and_warn(const char *parameter, bool keep) {
                 return 0;
         }
 
+        if (!reboot_parameter_is_valid(parameter))
+                return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Invalid reboot parameter '%s'.", parameter);
+
         WITH_UMASK(0022) {
                 r = write_string_file("/run/systemd/reboot-param", parameter,
                                       WRITE_STRING_FILE_CREATE|WRITE_STRING_FILE_ATOMIC);
