@@ -45,7 +45,7 @@ static char* const* const sleep_default_mode_table[_SLEEP_OPERATION_CONFIG_MAX]
 #if 0 /// elogind supports suspend modes (deep s2idle) so we need defaults, too
         /* Not used by SLEEP_SUSPEND */
 #else // 0
-        [SLEEP_SUSPEND]      = STRV_MAKE("s2idle", "deep"),
+        [SLEEP_SUSPEND]      = STRV_MAKE("deep", "s2idle"),
 #endif // 0
         [SLEEP_HIBERNATE]    = STRV_MAKE("platform", "shutdown"),
         [SLEEP_HYBRID_SLEEP] = STRV_MAKE("suspend"),
@@ -472,16 +472,24 @@ static int sleep_supported_internal(
                 }
 
                 r = hibernation_is_safe();
-                if (r == -ENOTRECOVERABLE) {
+                switch (r) {
+
+                case -ENOTRECOVERABLE:
                         *ret_support = SLEEP_RESUME_NOT_SUPPORTED;
                         return false;
-                }
-                if (r == -ENOSPC) {
+
+                case -ESTALE:
+                        *ret_support = SLEEP_RESUME_DEVICE_MISSING;
+                        return false;
+
+                case -ENOSPC:
                         *ret_support = SLEEP_NOT_ENOUGH_SWAP_SPACE;
                         return false;
+
+                default:
+                        if (r < 0)
+                                return r;
                 }
-                if (r < 0)
-                        return r;
 #if 0 /// elogind does support setting suspend modes
         } else
                 assert(!sleep_config->modes[operation]);
