@@ -11,11 +11,12 @@
 #define SET_IF(field, condition, value, fallback)  \
         field = (condition) ? (value) : (fallback)
 
-static inline const char* utf8_only(const char *s) {
+static const char* utf8_only(const char *s) {
         return s && utf8_is_valid(s) ? s : NULL;
 }
 
-static inline int strv_extend_strv_utf8_only(char ***dst, char **src, bool filter_duplicates) {
+#if 0 /// UNNEEDED by elogind
+static int strv_extend_strv_utf8_only(char ***dst, char **src, bool filter_duplicates) {
         _cleanup_free_ char **t = NULL;
         size_t l, j = 0;
 
@@ -34,6 +35,7 @@ static inline int strv_extend_strv_utf8_only(char ***dst, char **src, bool filte
         t[j] = NULL;
         return strv_extend_strv(dst, t, filter_duplicates);
 }
+#endif // 0
 
 int nss_passwd_to_user_record(
                 const struct passwd *pwd,
@@ -44,7 +46,6 @@ int nss_passwd_to_user_record(
         int r;
 
         assert(pwd);
-        assert(ret);
 
         if (isempty(pwd->pw_name))
                 return -EINVAL;
@@ -161,7 +162,8 @@ int nss_passwd_to_user_record(
         hr->mask = USER_RECORD_REGULAR |
                 (!strv_isempty(hr->hashed_password) ? USER_RECORD_PRIVILEGED : 0);
 
-        *ret = TAKE_PTR(hr);
+        if (ret)
+                *ret = TAKE_PTR(hr);
         return 0;
 }
 
@@ -216,7 +218,6 @@ int nss_user_record_by_name(
         int r;
 
         assert(name);
-        assert(ret);
 
         for (;;) {
                 buf = malloc(buflen);
@@ -257,7 +258,8 @@ int nss_user_record_by_name(
         if (r < 0)
                 return r;
 
-        (*ret)->incomplete = incomplete;
+        if (ret)
+                (*ret)->incomplete = incomplete;
         return 0;
 }
 
@@ -273,8 +275,6 @@ int nss_user_record_by_uid(
         struct spwd spwd, *sresult = NULL;
         int r;
 
-        assert(ret);
-
         for (;;) {
                 buf = malloc(buflen);
                 if (!buf)
@@ -313,7 +313,8 @@ int nss_user_record_by_uid(
         if (r < 0)
                 return r;
 
-        (*ret)->incomplete = incomplete;
+        if (ret)
+                (*ret)->incomplete = incomplete;
         return 0;
 }
 
@@ -327,7 +328,6 @@ int nss_group_to_group_record(
         int r;
 
         assert(grp);
-        assert(ret);
 
         if (isempty(grp->gr_name))
                 return -EINVAL;
@@ -377,7 +377,8 @@ int nss_group_to_group_record(
         g->mask = USER_RECORD_REGULAR |
                 (!strv_isempty(g->hashed_password) ? USER_RECORD_PRIVILEGED : 0);
 
-        *ret = TAKE_PTR(g);
+        if (ret)
+                *ret = TAKE_PTR(g);
         return 0;
 }
 
@@ -432,7 +433,6 @@ int nss_group_record_by_name(
         int r;
 
         assert(name);
-        assert(ret);
 
         for (;;) {
                 buf = malloc(buflen);
@@ -472,7 +472,8 @@ int nss_group_record_by_name(
         if (r < 0)
                 return r;
 
-        (*ret)->incomplete = incomplete;
+        if (ret)
+                (*ret)->incomplete = incomplete;
         return 0;
 }
 
@@ -488,8 +489,6 @@ int nss_group_record_by_gid(
         struct sgrp sgrp, *sresult = NULL;
         int r;
 
-        assert(ret);
-
         for (;;) {
                 buf = malloc(buflen);
                 if (!buf)
@@ -527,7 +526,8 @@ int nss_group_record_by_gid(
         if (r < 0)
                 return r;
 
-        (*ret)->incomplete = incomplete;
+        if (ret)
+                (*ret)->incomplete = incomplete;
         return 0;
 }
 #endif // 0
