@@ -5,7 +5,7 @@
 /***
   This file is part of elogind.
 
-  Copyright 2017-2018 Sven Eden
+  Copyright 2017 Sven Eden
 
   elogind is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published by
@@ -23,6 +23,6 @@
 
 #include <logind.h>
 
-int do_sleep(Manager *m, const char *verb);
+int do_sleep(Manager *sleep_config, SleepOperation operation);
 
 #endif // ELOGIND_SRC_SLEEP_SLEEP_H_INCLUDED
