@@ -183,7 +183,7 @@ TEST(chase) {
 
         /* Paths underneath the "root" with different UIDs while using CHASE_SAFE */
 
-        if (geteuid() == 0) {
+        if (geteuid() == 0 && !userns_has_single_user()) {
                 p = strjoina(temp, "/user");
                 assert_se(mkdir(p, 0755) >= 0);
                 assert_se(chown(p, UID_NOBODY, GID_NOBODY) >= 0);
@@ -236,6 +236,12 @@ TEST(chase) {
         assert_se(streq(result, "/test-chase.fsldajfl"));
         result = mfree(result);
 
+        r = chase("/.path/with/dot", temp, CHASE_PREFIX_ROOT|CHASE_NONEXISTENT, &result, NULL);
+        assert_se(r == 0);
+        q = strjoina(temp, "/.path/with/dot");
+        assert_se(streq(result, q));
+        result = mfree(result);
+
         r = chase("/etc/machine-id/foo", NULL, 0, &result, NULL);
 #if 1 /// elogind supports setups, where the machine-id is in the dbus default path
         if (!IN_SET(r, -ENOTDIR, -ENOENT)) {
@@ -313,7 +319,7 @@ TEST(chase) {
         r = chase(p, NULL, 0, &result, NULL);
         assert_se(r == -ENOENT);
 
-        if (geteuid() == 0) {
+        if (geteuid() == 0 && !userns_has_single_user()) {
                 p = strjoina(temp, "/priv1");
                 assert_se(mkdir(p, 0755) >= 0);
 
