@@ -6,7 +6,7 @@
 #include "tests.h"
 
 TEST(cescape) {
-        _cleanup_free_ char *t;
+        _cleanup_free_ char *t = NULL;
 
         assert_se(t = cescape("abc\\\"\b\f\n\r\t\v\a\003\177\234\313"));
         assert_se(streq(t, "abc\\\\\\\"\\b\\f\\n\\r\\t\\v\\a\\003\\177\\234\\313"));
@@ -14,7 +14,7 @@ TEST(cescape) {
 
 #if 0 /// UNNEEDED by elogind
 TEST(xescape) {
-        _cleanup_free_ char *t;
+        _cleanup_free_ char *t = NULL;
 
         assert_se(t = xescape("abc\\\"\b\f\n\r\t\v\a\003\177\234\313", ""));
         assert_se(streq(t, "abc\\x5c\"\\x08\\x0c\\x0a\\x0d\\x09\\x0b\\x07\\x03\\x7f\\x9c\\xcb"));
@@ -31,7 +31,7 @@ static void test_xescape_full_one(bool eight_bits) {
         log_info("/* %s */", __func__);
 
         for (unsigned i = 0; i < 60; i++) {
-                _cleanup_free_ char *t, *q;
+                _cleanup_free_ char *t = NULL, *q = NULL;
 
                 assert_se(t = xescape_full("abc\\\"\b\f\n\r\t\v\a\003\177\234\313", "b", i, flags));
 
@@ -40,7 +40,7 @@ static void test_xescape_full_one(bool eight_bits) {
                 if (i >= full_fit)
                         assert_se(streq(t, escaped));
                 else if (i >= 3) {
-                        /* We need up to four columns, so up to three three columns may be wasted */
+                        /* We need up to four columns, so up to three columns may be wasted */
                         assert_se(strlen(t) == i || strlen(t) == i - 1 || strlen(t) == i - 2 || strlen(t) == i - 3);
                         assert_se(strneq(t, escaped, i - 3) || strneq(t, escaped, i - 4) ||
                                   strneq(t, escaped, i - 5) || strneq(t, escaped, i - 6));
@@ -61,13 +61,13 @@ static void test_xescape_full_one(bool eight_bits) {
         }
 }
 
-TEST(test_xescape_full) {
+TEST(xescape_full) {
         test_xescape_full_one(false);
         test_xescape_full_one(true);
 }
 
 TEST(cunescape) {
-        _cleanup_free_ char *unescaped;
+        _cleanup_free_ char *unescaped = NULL;
 
         assert_se(cunescape("abc\\\\\\\"\\b\\f\\a\\n\\r\\t\\v\\003\\177\\234\\313\\000\\x00", 0, &unescaped) < 0);
         assert_se(cunescape("abc\\\\\\\"\\b\\f\\a\\n\\r\\t\\v\\003\\177\\234\\313\\000\\x00", UNESCAPE_RELAX, &unescaped) >= 0);
@@ -135,7 +135,7 @@ TEST(cunescape) {
 
 #if 0 /// UNNEEDED by elogind
 static void test_shell_escape_one(const char *s, const char *bad, const char *expected) {
-        _cleanup_free_ char *r;
+        _cleanup_free_ char *r = NULL;
 
         assert_se(r = shell_escape(s, bad));
         log_debug("%s → %s (expected %s)", s, r, expected);
@@ -207,7 +207,7 @@ TEST(shell_maybe_quote) {
 #endif // 0
 
 static void test_quote_command_line_one(char **argv, const char *expected) {
-        _cleanup_free_ char *s;
+        _cleanup_free_ char *s = NULL;
 
         assert_se(s = quote_command_line(argv, SHELL_ESCAPE_EMPTY));
         log_info("%s", s);
@@ -229,7 +229,7 @@ TEST(quote_command_line) {
 
 #if 0 /// UNNEEDED by elogind
 static void test_octescape_one(const char *s, const char *expected) {
-        _cleanup_free_ char *ret;
+        _cleanup_free_ char *ret = NULL;
 
         assert_se(ret = octescape(s, strlen_ptr(s)));
         log_debug("octescape(\"%s\") → \"%s\" (expected: \"%s\")", strnull(s), ret, expected);
