@@ -7,12 +7,13 @@
 #include "stdio-util.h"
 #include "string-util.h"
 #include "strv.h"
-#include "time-util.h"
 #include "tests.h"
+#include "time-util.h"
 
 TEST(hashmap_replace) {
-        Hashmap *m;
-        char *val1, *val2, *val3, *val4, *val5, *r;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
+        _cleanup_free_ char *val1 = NULL, *val2 = NULL, *val3 = NULL, *val4 = NULL, *val5 = NULL;
+        char *r;
 
         m = hashmap_new(&string_hash_ops);
 
@@ -39,17 +40,11 @@ TEST(hashmap_replace) {
         hashmap_replace(m, "key 5", val5);
         r = hashmap_get(m, "key 5");
         assert_se(streq(r, "val5"));
-
-        free(val1);
-        free(val2);
-        free(val3);
-        free(val4);
-        free(val5);
-        hashmap_free(m);
 }
 
 TEST(hashmap_copy) {
-        Hashmap *m, *copy;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
+        _cleanup_hashmap_free_free_ Hashmap *copy = NULL;
         char *val1, *val2, *val3, *val4, *r;
 
         val1 = strdup("val1");
@@ -78,14 +73,11 @@ TEST(hashmap_copy) {
         assert_se(streq(r, "val3"));
         r = hashmap_get(copy, "key 4");
         assert_se(streq(r, "val4"));
-
-        hashmap_free_free(copy);
-        hashmap_free(m);
 }
 
 TEST(hashmap_get_strv) {
-        Hashmap *m;
-        char **strv;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
+        _cleanup_strv_free_ char **strv = NULL;
         char *val1, *val2, *val3, *val4;
 
         val1 = strdup("val1");
@@ -114,15 +106,11 @@ TEST(hashmap_get_strv) {
         assert_se(streq(strv[1], "val2"));
         assert_se(streq(strv[2], "val3"));
         assert_se(streq(strv[3], "val4"));
-
-        strv_free(strv);
-
-        hashmap_free(m);
 }
 
 #if 0 /// UNNEEDED by elogind
 TEST(hashmap_move_one) {
-        Hashmap *m, *n;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL, *n = NULL;
         char *val1, *val2, *val3, *val4, *r;
 
         val1 = strdup("val1");
@@ -155,13 +143,10 @@ TEST(hashmap_move_one) {
         assert_se(!r);
 
         assert_se(hashmap_move_one(n, m, "key 3") == -EEXIST);
-
-        hashmap_free_free(m);
-        hashmap_free_free(n);
 }
 
 TEST(hashmap_move) {
-        Hashmap *m, *n;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL, *n = NULL;
         char *val1, *val2, *val3, *val4, *r;
 
         val1 = strdup("val1");
@@ -197,15 +182,13 @@ TEST(hashmap_move) {
         assert_se(r && streq(r, "val3"));
         r = hashmap_get(n, "key 4");
         assert_se(r && streq(r, "val4"));
-
-        hashmap_free_free(m);
-        hashmap_free_free(n);
 }
 #endif // 0
 
 TEST(hashmap_update) {
-        Hashmap *m;
-        char *val1, *val2, *r;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
+        _cleanup_free_ char *val1 = NULL, *val2 = NULL;
+        char *r;
 
         m = hashmap_new(&string_hash_ops);
         val1 = strdup("old_value");
@@ -224,14 +207,10 @@ TEST(hashmap_update) {
         assert_se(hashmap_update(m, "key 1", val2) == 0);
         r = hashmap_get(m, "key 1");
         assert_se(streq(r, "new_value"));
-
-        free(val1);
-        free(val2);
-        hashmap_free(m);
 }
 
 TEST(hashmap_put) {
-        Hashmap *m = NULL;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
         int valid_hashmap_put;
         void *val1 = (void*) "val 1";
         void *val2 = (void*) "val 2";
@@ -247,8 +226,6 @@ TEST(hashmap_put) {
         key1 = strdup("key 1");
         assert_se(hashmap_put(m, key1, val1) == 0);
         assert_se(hashmap_put(m, key1, val2) == -EEXIST);
-
-        hashmap_free(m);
 }
 
 TEST(hashmap_remove1) {
@@ -444,7 +421,7 @@ TEST(hashmap_ensure_allocated) {
 }
 
 TEST(hashmap_foreach_key) {
-        Hashmap *m;
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
         bool key_found[] = { false, false, false, false };
         const char *s;
         const char *key;
@@ -456,8 +433,8 @@ TEST(hashmap_foreach_key) {
 
         m = hashmap_new(&string_hash_ops);
 
-        NULSTR_FOREACH(key, key_table)
-                hashmap_put(m, key, (void*) (const char*) "my dummy val");
+        NULSTR_FOREACH(k, key_table)
+                hashmap_put(m, k, (void*) (const char*) "my dummy val");
 
         HASHMAP_FOREACH_KEY(s, key, m) {
                 assert_se(s);
@@ -473,12 +450,10 @@ TEST(hashmap_foreach_key) {
 
         assert_se(m);
         assert_se(key_found[0] && key_found[1] && key_found[2] && !key_found[3]);
-
-        hashmap_free(m);
 }
 
 TEST(hashmap_foreach) {
-        Hashmap *m;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
         bool value_found[] = { false, false, false, false };
         char *val1, *val2, *val3, *val4, *s;
         unsigned count;
@@ -492,8 +467,6 @@ TEST(hashmap_foreach) {
         val4 = strdup("my val4");
         assert_se(val4);
 
-        m = NULL;
-
         count = 0;
         HASHMAP_FOREACH(s, m)
                 count++;
@@ -524,12 +497,11 @@ TEST(hashmap_foreach) {
 
         assert_se(m);
         assert_se(value_found[0] && value_found[1] && value_found[2] && value_found[3]);
-
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_merge) {
-        Hashmap *m, *n;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
+        _cleanup_hashmap_free_ Hashmap *n = NULL;
         char *val1, *val2, *val3, *val4, *r;
 
         val1 = strdup("my val1");
@@ -557,12 +529,10 @@ TEST(hashmap_merge) {
 
         assert_se(m);
         assert_se(n);
-        hashmap_free(n);
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_contains) {
-        Hashmap *m;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
         char *val1;
 
         val1 = strdup("my val");
@@ -578,11 +548,10 @@ TEST(hashmap_contains) {
         assert_se(!hashmap_contains(NULL, "Key 1"));
 
         assert_se(m);
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_isempty) {
-        Hashmap *m;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
         char *val1;
 
         val1 = strdup("my val");
@@ -595,11 +564,10 @@ TEST(hashmap_isempty) {
         assert_se(!hashmap_isempty(m));
 
         assert_se(m);
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_size) {
-        Hashmap *m;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
         char *val1, *val2, *val3, *val4;
 
         val1 = strdup("my val");
@@ -624,11 +592,10 @@ TEST(hashmap_size) {
         assert_se(m);
         assert_se(hashmap_size(m) == 4);
         assert_se(hashmap_buckets(m) >= 4);
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_get) {
-        Hashmap *m;
+        _cleanup_hashmap_free_free_ Hashmap *m = NULL;
         char *r;
         char *val;
 
@@ -649,11 +616,10 @@ TEST(hashmap_get) {
         assert_se(r == NULL);
 
         assert_se(m);
-        hashmap_free_free(m);
 }
 
 TEST(hashmap_get2) {
-        Hashmap *m;
+        _cleanup_(hashmap_free_free_freep) Hashmap *m = NULL;
         char *r;
         char *val;
         char key_orig[] = "Key 1";
@@ -682,7 +648,6 @@ TEST(hashmap_get2) {
         assert_se(r == NULL);
 
         assert_se(m);
-        hashmap_free_free_free(m);
 }
 
 static void crippled_hashmap_func(const void *p, struct siphash *state) {
@@ -1006,6 +971,44 @@ TEST(string_strv_hashmap) {
 }
 #endif // 0
 
+TEST(hashmap_dump_sorted) {
+        static void * const expected[] = { UINT_TO_PTR(123U), UINT_TO_PTR(12U), UINT_TO_PTR(345U), };
+        _cleanup_hashmap_free_ Hashmap *m = NULL;
+        _cleanup_free_ void **vals = NULL;
+        size_t n;
+
+        assert_se(m = hashmap_new(&string_hash_ops));
+
+        assert_se(hashmap_dump_sorted(m, &vals, &n) >= 0);
+        assert_se(n == 0);
+        assert_se(!vals);
+
+        assert_se(hashmap_put(m, "key 0", expected[0]) == 1);
+        assert_se(hashmap_put(m, "key 1", expected[1]) == 1);
+        assert_se(hashmap_put(m, "key 2", expected[2]) == 1);
+
+        assert_se(hashmap_dump_sorted(m, &vals, &n) >= 0);
+        assert_se(n == ELEMENTSOF(expected));
+        assert_se(memcmp(vals, expected, n * sizeof(void*)) == 0);
+
+        vals = mfree(vals);
+        m = hashmap_free(m);
+
+        assert_se(m = hashmap_new(NULL));
+
+        assert_se(hashmap_dump_sorted(m, &vals, &n) >= 0);
+        assert_se(n == 0);
+        assert_se(!vals);
+
+        assert_se(hashmap_put(m, UINT_TO_PTR(333U), expected[2]) == 1);
+        assert_se(hashmap_put(m, UINT_TO_PTR(222U), expected[1]) == 1);
+        assert_se(hashmap_put(m, UINT_TO_PTR(111U), expected[0]) == 1);
+
+        assert_se(hashmap_dump_sorted(m, &vals, &n) >= 0);
+        assert_se(n == ELEMENTSOF(expected));
+        assert_se(memcmp(vals, expected, n * sizeof(void*)) == 0);
+}
+
 /* Signal to test-hashmap.c that tests from this compilation unit were run. */
 extern int n_extern_tests_run;
 TEST(ensure_extern_hashmap_tests) {
