@@ -62,7 +62,7 @@ TEST(terminal_urlify) {
 #if 0 /// UNNEEDED by elogind
 TEST(cat_files) {
         assert_se(cat_files("/no/such/file", NULL, 0) == -ENOENT);
-        assert_se(cat_files("/no/such/file", NULL, CAT_FLAGS_MAIN_FILE_OPTIONAL) == 0);
+        assert_se(cat_files(NULL, NULL, 0) == 0);
 
         if (access("/etc/fstab", R_OK) >= 0)
                 assert_se(cat_files("/etc/fstab", STRV_MAKE("/etc/fstab", "/etc/fstab"), 0) == 0);
