@@ -1,12 +1,12 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
-#if 0 /// UNNEEDED by elogind (only needed for testing __cpu_mask)
 #include <sched.h>
-#endif // 0
 #include <stdio.h>
 #include <string.h>
-#include <sys/types.h>
+#include <sys/resource.h>
 #include <sys/socket.h>
+#include <sys/timex.h>
+#include <sys/types.h>
 
 #define __STDC_WANT_IEC_60559_TYPES_EXT__
 #include <float.h>
@@ -19,16 +19,28 @@
 DISABLE_WARNING_TYPE_LIMITS;
 
 #define info_no_sign(t)                                                 \
-        printf("%s → %zu bits, %zu byte alignment\n", STRINGIFY(t),     \
+        printf("%s → %zu bits, %zu byte alignment\n", STRINGIFY(t),    \
                sizeof(t)*CHAR_BIT,                                      \
-               __alignof__(t))
+               alignof(t))
 
 #define info(t)                                                         \
-        printf("%s → %zu bits%s, %zu byte alignment\n", STRINGIFY(t),   \
+        printf("%s → %zu bits%s, %zu byte alignment\n", STRINGIFY(t),  \
                sizeof(t)*CHAR_BIT,                                      \
                strstr(STRINGIFY(t), "signed") ? "" :                    \
                (t)-1 < (t)0 ? ", signed" : ", unsigned",                \
-               __alignof__(t))
+               alignof(t))
+
+#define check_no_sign(t, size)                  \
+        do {                                    \
+                info_no_sign(t);                \
+                assert_se(sizeof(t) == size);   \
+        } while (false)
+
+#define check(t, size)                          \
+        do {                                    \
+                info(t);                        \
+                assert_se(sizeof(t) == size);   \
+        } while (false)
 
 enum Enum {
         enum_value,
@@ -46,7 +58,13 @@ enum BigEnum2 {
 int main(void) {
         int (*function_pointer)(void);
 
-        info_no_sign(function_pointer);
+        check_no_sign(dev_t, SIZEOF_DEV_T);
+        check_no_sign(ino_t, SIZEOF_INO_T);
+        check_no_sign(rlim_t, SIZEOF_RLIM_T);
+        check(time_t, SIZEOF_TIME_T);
+        check(typeof(((struct timex *)0)->freq), SIZEOF_TIMEX_MEMBER);
+
+        info_no_sign(typeof(function_pointer));
         info_no_sign(void*);
         info(char*);
 
@@ -55,12 +73,12 @@ int main(void) {
         info(unsigned char);
         info(short unsigned);
         info(unsigned);
-#ifdef __GLIBC__
         info(unsigned long);
         info(unsigned long long);
+#ifdef __GLIBC__
         info(__syscall_ulong_t);
         info(__syscall_slong_t);
-#endif // ifdef __GLIBC__
+#endif
         info(intmax_t);
         info(uintmax_t);
 
@@ -78,19 +96,18 @@ int main(void) {
 
         info(size_t);
         info(ssize_t);
-        info(time_t);
         info(usec_t);
 #ifdef __GLIBC__
         info(__time_t);
-#endif // ifdef __GLIBC__
+#endif
         info(pid_t);
         info(uid_t);
         info(gid_t);
         info(socklen_t);
 
-#if 0 /// UNNEEDED by elogind (Only needed for *cpu_set* stuff in systemd we don't use)
+#ifdef __GLIBC__
         info(__cpu_mask);
-#endif // 0
+#endif
 
         info(enum Enum);
         info(enum BigEnum);
