@@ -23,6 +23,10 @@
         "non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
 
 #if 0 /// Nowhere used in elogind
+TEST(colors_enabled) {
+        log_info("colors_enabled: %s", yes_no(colors_enabled()));
+}
+
 TEST(default_term_for_tty) {
         puts(default_term_for_tty("/dev/tty23"));
         puts(default_term_for_tty("/dev/ttyS23"));
